#!/usr/bin/env python

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os.path
import random

import events
import item
import slist

from helper import debug

class filelist(slist.slist):

    def __init__(self, win, mp3basedir, songdb):
        slist.slist.__init__(self, win)

        self.mp3basedir = mp3basedir
        self.songdb = songdb
        self.basedir = item.basedir(mp3basedir, win.ih)
        self.dir = [self.basedir]
        self.shistory = []
        self.readdir()

        self.win.channel.subscribe(events.artistaddedtosongdb,
                                   self.artistaddedtosongdb)
        self.win.channel.subscribe(events.albumaddedtosongdb,
                                   self.albumaddedtosongdb)

    def getselected(self):
        return slist.slist.getselected(self)

    def isdirselected(self):
        return isinstance(slist.slist.getselected(self), item.diritem)

    def issongselected(self):
        return isinstance(slist.slist.getselected(self), item.song)
        
    def getselectedsubdir(self):
        return self.dir + [self.getselected()]

    def readdir(self):
        self.set(self.dir[-1].getcontents(self.songdb))
        return 1

    def dirdown(self):
        self.shistory.append((self.dir, self.selected, self.top))
        self.dir = self.getselectedsubdir()
        
        if self.readdir() is None:
            self.dir, self.selected, self.top = self.shistory.pop()
            self.readdir()

    def dirup(self):
        if len(self.shistory)>0:
            dir, selected, top = self.shistory.pop()
            self.dir = dir
            self.readdir()
            self.selected = selected
            self.top = top
            events.hub.notify(events.selectionchanged(self.getselected()))

    def jumptorandomlist(self):
        if self.dir != [self.basedir, self.basedir.randomlist]:
            self.shistory.append((self.dir, self.selected, self.top))
        self.dir = [self.basedir, self.basedir.randomlist]
        self.readdir()

    def jumptofilesystembase(self):
        if self.dir != [self.basedir, self.basedir.filesystembase]:
            self.shistory.append((self.dir, self.selected, self.top))
        self.dir = [self.basedir, self.basedir.filesystembase]
        self.readdir()
        
    def selectionpath(self):
        return self.dir[-1].getheader(self.getselected(), self.songdb)
        
    def insertrecursiveselection(self):
        self.insertrecursive(self.getselected())

    def insertrecursive(self, dir):
        insertlist = dir.getcontents(self.songdb)
        for aitem in insertlist:
            if isinstance(aitem, item.diritem):
                self.insertrecursive(aitem)
            elif isinstance(aitem, item.song):
                events.hub.notify(events.playlistaddsong(aitem))

    # event handler

    def artistaddedtosongdb(self, event):
        if isinstance(self.dir[-1], item.basedir):
            self.insertitem(event.artist, self.dir[-1].cmpitem)
            self.win.update()

    def albumaddedtosongdb(self, event):
        if (isinstance(self.dir[-1], item.artist) and 
            self.dir[-1].name==event.album.artist):
            self.insertitem(event.album, self.dir[-1].cmpitem)
            self.win.update()
