#!/usr/bin/env python

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import curses
import time

import item
import events
import window

from helper import formattime

class iteminfowin(window.window):
    def __init__(self, screen, h, w, y, x, colors, channel, player):
        self.item = None
        self.player = player
        channel.subscribe(events.selectionchanged, self.selectionchanged)
        window.window.__init__(self, screen, h, w, y, x,
                               colors,
                               _("MP3 Info"))

    def selectionchanged(self, event):
        self.item = event.item
        if isinstance(self.item, item.song):
            events.hub.notify(events.nextsong(self.player, self.item))
        self.update()

    def update(self):
        window.window.update(self)

        # lines to display
        l = [ ["", "", "", ""] for i in range(4)]
        
        if isinstance(self.item, item.song):
            l[0] = [_("Title:"), self.item.title]
            if self.item.tracknr:
                l[0] += [_("Nr:"), str(self.item.tracknr)]
            else:
                l[0] += ["", ""]
            l[1] = [_("Album:"),  self.item.album]
            if self.item.year!="" and self.item.year!="0":
                l[1] += [_("Year:"), self.item.year]
            else:
                l[1] += ["", ""]
            l[2] = [_("Artist:"), self.item.artist,
                  _("Time:"), formattime(self.item.length/1000)]
            l[3] = [_("Genre:"), self.item.genre]
            if self.item.nrplayed>0:
                last = int((time.time()-self.item.lastplayed)/60)
                days, rest = divmod(last, 24*60)
                hours, minutes = divmod(rest, 24)
                if days>0:
                    lastplayed = "%dd%dh%dm" % (days, hours, minutes)
                elif hours>0:
                    lastplayed = "%dh%dm" % (hours, minutes)
                else:
                    lastplayed = "%dm" % minutes
                l[3] += [_("Played:"),
                       _("#%d, %s ago") % (self.item.nrplayed, lastplayed)]
            else:
                l[3] += ["", ""]
        elif isinstance(self.item, item.artist):
            l[0] = [_("Artist:"), self.item.name, "", ""]
        elif isinstance(self.item, item.album):
            l[0] = [_("Artist:"), self.item.artist, "", ""]
            l[1] = [_("Album:"), self.item.name, "", ""]
        elif isinstance(self.item, item.filtereditem):
            if isinstance(self.item.item, item.artist):
                l[0] = [_("Artist:"), self.item.item.name, "", ""]
                l[1] = [_("Filter:"), self.item.fname, "", ""]
            if isinstance(self.item.item, item.album):
                l[0] = [_("Artist:"), self.item.item.artist, "", ""]
                l[1] = [_("Album:"), self.item.item.name, "", ""]
                l[2] = [_("Filter:"), self.item.fname, "", ""]
        elif isinstance(self.item, item.genre):
            l[0] = ["%s:" % _("Genre"), self.item.name, "", ""]
        elif isinstance(self.item, item.playlist):
            l[0] = ["%s:" % _("Playlist"), self.item.name, "", ""]
        elif isinstance(self.item, item.filesystemdir):
            l[0] = ["%s:" % _("Filesystem"), self.item.dir, "", ""]
        elif isinstance(self.item, item.genres):
            l[0] = [_("Genres"), "", "", ""]
        elif isinstance(self.item, item.topplayed):
            l[0] = [_("Top played songs"), "", "", ""]
        elif isinstance(self.item, item.lastplayed):
            l[0] = [_("Last played songs"), "", "", ""]
        elif isinstance(self.item, item.randomlist):
            l[0] = [_("Random song list"), "", "", ""]
        elif isinstance(self.item, item.playlists):
            l[0] = [_("Playlists"), "", "", ""]

        colsep = self.iw>45

        # calculate width of columns
        wc1 = max( len(l[0][0]), len(l[1][0]), len(l[2][0]), len(l[3][0])) + colsep
        wc3 = max( len(l[0][2]), len(l[1][2]), len(l[2][2])) + colsep
        wc4 = 5
        wc2 = self.iw-wc1-wc3-wc4-1

        for lno in range(4):
            self.win.move(1+lno, 1)
            self.win.addstr(l[lno][0].ljust(wc1)[:wc1], self.colors["description"])
            self.win.addstr(l[lno][1].ljust(wc2)[:wc2], self.colors["content"])
            self.win.addch(" ")
            if lno!=3:
                self.win.addstr(l[lno][2].ljust(wc3)[:wc3], self.colors["description"])
                self.win.addstr(l[lno][3].ljust(wc4)[:wc4], self.colors["content"])
            else:
                # special handling of last line
                wc3 = max(len(l[3][2]), 5) + colsep
                wc4 = max(len(l[3][3]), 5)
                
                self.win.move(1+lno, self.w-wc3-wc4-2)
                self.win.addch(" ")
                self.win.addstr(l[3][2].ljust(wc3)[:wc3], self.colors["description"])
                self.win.addstr(l[3][3].ljust(wc4)[:wc4], self.colors["content"])
        
