#!/usr/bin/env python

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import curses
import curses.panel
from helper import debug
import events

class window:
    def __init__(self, screen,
                 h, w, y, x,
                 colors,
                 title=None, hasscrollbar=0):
        self.screen = screen
        self.win = curses.newwin(h, w, y, x)
        self.panel = curses.panel.new_panel(self.win)
        curses.panel.update_panels()
        self.colors = colors
        self.hasscrollbar = hasscrollbar
        self._setdimensions(h, w, y, x)
        self.settitle(title)

    def _setdimensions(self, h, w, y, x):
        self.h = h
        self.w = w
        self.x = x
        self.y = y


        self.win.bkgdset(0, self.colors["background"])

        # coordinates of upper left corner and width and height of inner part of window
        if self.h>2:
            # we have a border around the window
            self.ix = 1
            self.iy = 1
            self.ih = h-2
            self.iw = w-2
        else:
            self.ix = 0
            self.iy = 0
            self.ih = h
            self.iw = w

        if self.hasscrollbar:
            self.iw -= 1

    def resize(self, h, w, y, x):
        """ resize window """
        try:
            self.win.resize(h, w)
            self.win.mvwin(y, x)
        except curses.error:
            pass
        self._setdimensions(h, w, y, x)

    def settitle(self, title):
        if title is not None:
            self.title = title[:self.w-8]
        else:
            self.title = None

    def drawscrollbar(self, top, total):
        if self.hasscrollbar:
            height = self.ih
            xpos = self.iw+self.ix

            self.win.addch(1, xpos,
                           top!=0 and curses.ACS_UARROW or " ",
                           self.colors["scrollbararrow"])
            self.win.addch(height, xpos,
                           top+height<total and curses.ACS_DARROW or " ",
                           self.colors["scrollbararrow"])
        
            if total>height:
                scrollbarbegin = 2 + height*top/total
                scrollbarend = scrollbarbegin + height*height/total-2
                
                for line in range(2, height):
                    if line>=scrollbarbegin and line<=scrollbarend:
                        attr = self.colors["scrollbarhigh"]
                    else:
                        attr = self.colors["scrollbar"]
                    self.win.addch(line, xpos, "#", attr)
            else:
                for line in range(2, height):
                    self.win.addch(line, xpos, " ", self.colors["background"])
            

    def top(self):
        """ bring window to top"""
        try: self.panel.top()
        except: pass
        else: events.hub.notify(events.focuschanged())

    def hide(self):
        """ hide window """
        try: self.panel.hide()
        except: pass
        else: events.hub.notify(events.focuschanged())

    def hasfocus(self):
        """ is window on top and has thus the current focus? """
        return self.panel is curses.panel.top_panel()

    def enclose(self, y, x):
        "check whether y, x belongs to window (and not to other windows lying above)"

        # walk through panel stack
        p = curses.panel.top_panel()
        while p:
            if p is self.panel:
                return self.win.enclose(y, x)
            elif p.window().enclose(y, x):
                return 0
            p = p.below()

    def stdscrtowin(self, y, x):
        begy, begx = self.win.getbegyx()
        return y-begy, x-begx
    
    def update(self):
        if self.title and self.h>2:
            if self.hasfocus():
                topborder = ord("=")
                self.win.attrset(self.colors["activeborder"])
            else:
                topborder = 0
                self.win.attrset(self.colors["border"])
            self.win.border(0, 0, topborder)
            self.win.attrset(0)
            t = self.title[:self.w-4]
            pos = (self.w-4-len(t))/2 
            self.win.addstr(0, pos, "[ %s ]" % t, self.colors["title"])

    # event handler

    def hidewindow(self, event):
        # subclasses of window explicitely have to subscribe to this event,
        # if they need to
        if event.window==self:
            self.hide()
